# load the packages
library(tidyverse)
library(foreign) ; library(tsModel) ; library(lmtest) ; library(Epi)
library(splines) ; library(vcd) ; library(gplots) ; library(car) ; library(plm) ; library(readxl)

# read data from excel file
X27zipcodes <- read_excel("E:/My Drive/PhD/Publications/San Francisco/Data/27zipcodes.xls", 
                            +     col_types = c("numeric", "numeric", "numeric", 
                                                +         "numeric", "numeric", "numeric", 
                                                +         "numeric", "numeric", "numeric", 
                                                +         "numeric", "numeric"))
data <- X27zipcodes
data$date <- sprintf("%d-%02d", data$yeah, data$month)

# categorize strratio into 11 levels
data$ratecat <- cut(data$strratio, seq(0,0.06,0.01), right=FALSE, labels=c(1:6))

# plot the prices with the ratecat
ggplot(data = data, mapping = aes(x = factor(time), y = price, color = ratecat)) +
    geom_point(alpha = 0.7, size = 3) + 
    labs(x = "Year",
         y = "Home Price Per Square Foot in USD",
         color = "STR Density") +
    scale_x_discrete(breaks=c(9, 21, 33, 45),
                     labels=c("2016", "2017", "2018", "2019")) +
    theme(
        # change background color
        plot.background = element_rect(fill = "deepskyblue4"),
        # get rid of panel grids
        panel.grid.major = element_line("darkslategray4"),
        panel.grid.minor = element_line("darkslategray4"),
        # Change panel background
        panel.background = element_rect(fill = 'deepskyblue4'),
        # change title, label and axis colors
        axis.text = element_text(color= "white", size = 7),
        axis.title.x = element_text(colour = "white", face = "bold"),
        axis.title.y = element_text(colour = "white", face = "bold"),
        # Change legend 
        legend.position = c(0.82, 0.08),
        legend.direction = "horizontal",
        legend.background = element_rect(fill = "transparent", color = NA),
        legend.key = element_rect(color = "deepskyblue4", fill = "deepskyblue4"),
        legend.title = element_text(color = "white"),
        legend.text = element_text(color = "white")
    ) +
    guides(size = FALSE)

# plot STR density across zip codes
par(bg = "deepskyblue4")
plotmeans(strratio ~ id,
          connect = FALSE,
          n.label = FALSE,
          xlab = "Zip Code",
          ylab = "STR Density",
          cex.axis = 0.7,
          barwidth = 2,
          lwd = 3,
          pch = 16,
          size = 3,
          main="Mean STR density across 27 zip codes",
          data=data,
          col = "darkorange",
          col.axis = "white",
          col.main = "white",
          col.lab = "white",
          barcol = "slategray3",
          ccol = "darkorange1",
          border = "white")

# plot STR density across time
par(bg = "deepskyblue4")
plotmeans(strratio ~ date,
          n.label = FALSE,
          xlab = "Date",
          ylab = "STR Density",
          cex.axis = 0.7,
          barwidth = 2,
          lwd = 3,
          main="Mean STR density over time",
          data=data,
          col = "slategray1",
          col.axis = "white",
          col.main = "white",
          col.lab = "white",
          barcol = "slategray3",
          ccol = "darkorange1",
          border = "white")

# Fixed effects using Least squares dummy variable model
fixed.dum <- lm(strratio ~ regulation + time + regulation*time + factor(id) - 1, na.action = na.exclude, data=data)
summary(fixed.dum)
data$yhat <- fitted(fixed.dum)
scatterplot(data$yhat~data$regulation|data$id, boxplots=FALSE, xlab="x1", ylab="yhat",smooth=FALSE, na.action=na.exclude, data=data)

# alternative method
datanew <- data[,c(1,4,5,6,7,8,9,10,11,12,13,14)]

pool <- plm(strratio ~ regulation, na.action = na.exclude, data = datanew, model = "pooling")
summary(pool)
pool2 <- plm(strratio ~ price + regulation, na.action = na.exclude, data = datanew, model = "pooling")
summary(pool2)
pool3 <- plm(strratio ~ time, na.action = na.exclude, data = datanew, model = "pooling")
summary(pool3)
pool4 <- plm(strratio ~ regulation + price + time, na.action = na.exclude, data = datanew, model = "pooling")
summary(pool4)
pool5 <- plm(strratio ~ regulation + time, na.action = na.exclude, data = datanew, model = "pooling")
summary(pool5)

fxd <- plm(strratio ~ regulation, na.action = na.exclude, data = datanew, model = "within")
summary(fxd)
fxd2 <- plm(strratio ~ price + regulation, na.action = na.exclude, data = datanew, model = "within")
summary(fxd2)
fxd3 <- plm(strratio ~ time, na.action = na.exclude, data = datanew, model = "within")
summary(fxd3)
fxd4 <- plm(strratio ~ regulation + price + time, na.action = na.exclude, data = datanew, model = "within")
summary(fxd4)
fxd5 <- plm(strratio ~ regulation + time, na.action = na.exclude, data = datanew, model = "within")
summary(fxd5)

# random effect
ran <- plm(strratio ~ regulation, na.action = na.exclude, data = datanew, model = "random")
summary(ran)
ran2 <- plm(strratio ~ price + regulation, na.action = na.exclude, data = datanew, model = "random")
summary(ran2)
ran3 <- plm(strratio ~ time, na.action = na.exclude, data = datanew, model = "random")
summary(ran3)
ran4 <- plm(strratio ~ regulation + price + time, na.action = na.exclude, data = datanew, model = "random")
summary(ran4)
ran5 <- plm(strratio ~ regulation + time, na.action = na.exclude, data = datanew, model = "random")
summary(ran5)

# LM test between OLS and Random Effect
plmtest(pool)
plmtest(pool2)
plmtest(pool3)
plmtest(pool4)
plmtest(pool5)

# LM test between OLS and Fixed Effect
pFtest(fxd,pool)
pFtest(fxd2,pool2)
pFtest(fxd3,pool3)
pFtest(fxd4,pool4)
pFtest(fxd5,pool5)

# HOuseman Test
phtest(ran,fxd)
phtest(ran2,fxd2)
phtest(ran3,fxd3)
phtest(ran4,fxd4)
phtest(ran5,fxd5)
# p-value of the test is 1. Choose random effect modeling

