# load the packages
library(foreign) ; library(tsModel) ; library("lmtest") ; library("Epi")
library("splines") ; library("vcd")

# read data from excel file
sf <- read_excel("E:/My Drive/PhD/Publications/San Francisco/Data/sf.xlsx", 
                 +     col_types = c("numeric", "numeric", "numeric", 
                                     +         "numeric", "numeric", "numeric", 
                                     +         "numeric", "numeric", "numeric", 
                                     +         "numeric"))
head(data)
View(data)
data <- sf

# histogram (confirmation of poisson distribution)
hist(data$str)

# plot strratio
data$rate <- with(data, strratio)
plot(data$rate,type="n",ylim=c(00,0.05),xlab="Year", ylab="Str ratio", bty="l",xaxt="n")
rect(33,0,47,0.05,col=grey(0.9),border=F)
points(data$rate[data$regulation==0],cex=0.7)
ticks <- c(9, 21, 33, 45)
axis(1, at = ticks, labels = F, tick = TRUE)
years <- c(3, 15, 27, 39)
axis(1,at=years,tick=F,labels=2015:2018)
title("San Francisco, May 2015 - March 2019")

# modeling

# simple poisson regression of str to regulation and time
m1 <- glm(str ~ regulation + time, family = poisson, data)
summary(m1)
summary(m1)$dispersion
round(ci.lin(m1,Exp=T),3)

# create a new dataframe with 0.1 time units to improve the graph
x = 51:120/10
y = 1:50/10
v = 1:40/10
w = c(x,y,x,y,x,y,x,v)
datanew <- data.frame(houses=mean(data$houses),regulation=rep(c(0,1),c(320,150)),
                      time= 1:470/10, month=w)

# generate predicted values based on the model in order to create a plot
pred1 <- predict(m1,type="response",datanew)/mean(data$houses)

#This can then be plotted along with a scatter graph (see above)
plot(data$rate,type="n",ylim=c(00,0.03),xlab="Year",ylab="Str ratio",
     bty="l",xaxt="n")
rect(33,0,47,0.05,col=grey(0.9),border=F)
points(data$rate,cex=0.7)
axis(1, at = ticks, labels = F, tick = TRUE)
axis(1,at=years,tick=F,labels=2015:2018)
lines((1:470/10),pred1,col=2)
title("San Francisco, May 2015 - March 2019")

# to plot the counterfactual scenario, create a data frame as if regulation
#   (the intervention) was never being implemented
datanew <- data.frame(houses=mean(data$houses),regulation=0,time=1:470/10, month=w)

# generate predictions under the counterfactual scenario and add it to the plot
pred1b <- predict(m1,datanew,type="response")/mean(data$houses)
lines(datanew$time,pred1b,col=2,lty=2)

# return the data frame to the scenario including the intervention
datanew <- data.frame(houses=mean(data$houses),regulation=rep(c(0,1),c(320,150)),
                      time= 1:470/10, month=w)

# tackle overdispersion
m2 <- glm(str ~ regulation + time, family = quasipoisson, data)
summary(m2)
summary(m2)$dispersion
round(ci.lin(m2,Exp=T),3)

# check residuals
res2 <- residuals(m2,type="deviance")
length(res2) = length(data$time)
plot(data$time,res2,ylim=c(-45,95),pch=19,cex=0.7,col=grey(0.6), main="Residuals over time",
     ylab="Deviance residuals",xlab="Date")
abline(h=0,lty=2,lwd=2)

# check for autocorrelation
res2 <- residuals(m2,type="deviance")
acf(res2)

# adjusting for seasonality
m3 <- glm(str ~ regulation + time + harmonic(month,2,12), family = quasipoisson, data)
summary(m3)
summary(m3)$dispersion
round(ci.lin(m3,Exp=T),3)

# effect of regulation
ci.lin(m3,Exp=T)["regulation",1:7]

# trend
exp(coef(m3)["time"]*12)

# check the model and autocorrelation
res3 <- residuals(m3,type="deviance")
plot(res3,ylim=c(-45,95),pch=19,cex=0.7,col=grey(0.6),main="Residuals over time",
     ylab="Deviance residuals",xlab="Date")
abline(h=0,lty=2,lwd=2)
acf(res3)
pacf(res3)

# predict and plot of the seasonally adjusted model
pred3 <- predict(m3,type="response",datanew)/mean(data$houses)
plot(data$rate,type="n",ylim=c(00,0.03),xlab="Year",ylab="Str ratio",
     bty="l",xaxt="n")
rect(33,0,47,0.05,col=grey(0.9),border=F)
points(data$rate,cex=0.7)
axis(1, at = ticks, labels = F, tick = TRUE)
axis(1,at=years,tick=F,labels=2015:2018)
lines((1:470/10),pred3,col=2)
title("San Francisco, May 2015 - March 2019")

# plot a straight line representing a 'deseasonalised' trend by predicting all the observations for June
pred3b <- predict(m3,type="response",transform(datanew,month=6))/
    mean(data$houses)
lines(1:470/10,pred3b,col=2,lty=2)

# parameterize it as an interaction between time and the ban indicator
m4 <- glm(str ~ regulation*time + harmonic(month,2,12),
              family=quasipoisson, data)
summary(m4)
round(ci.lin(m4,Exp=T),3)

# predict and plot the 'deseasonalised' trend and compare it with the step-change only model
pred4b <- predict(m4,type="response",transform(datanew,month=6))/
    mean(data$houses)
plot(data$rate,type="n",ylim=c(00,0.03),xlab="Year",ylab="Str ratio",
     bty="l",xaxt="n")
rect(33,0,47,0.05,col=grey(0.9),border=F)
points(data$rate,cex=0.7)
axis(1, at = ticks, labels = F, tick = TRUE)
axis(1,at=years,tick=F,labels=2015:2018)
lines((1:470/10),pred3b,col=2)
lines((1:470/10),pred4b,col=4)
title("San Francisco, May 2015 - March 2019")
legend("topleft",c("Step-change only","Step-change + change-in-slope"),lty=1,
       col=c(2,4),inset=0.05,bty="n",cex=0.7)

# test if the change-in-slope improve the fit
# the selected test here is an F-test, which accounts for the overdispersion,
#   while in other cases a likelihood ratio or wald test can be applied
anova(m3,m4,test="F")
# not surprisingly, the p-value is similar to that of the interaction term

# predict and plot of the seasonally adjusted version of m4
pred4 <- predict(m4,type="response",datanew)/mean(data$houses)
plot(data$rate,type="n",ylim=c(00,0.03),xlab="Year",ylab="Str ratio",
     bty="l",xaxt="n")
rect(33,0,47,0.05,col=grey(0.9),border=F)
points(data$rate,cex=0.7)
axis(1, at = ticks, labels = F, tick = TRUE)
axis(1,at=years,tick=F,labels=2015:2018)
lines((1:470/10),pred4,col=2)
title("San Francisco, May 2015 - March 2019")

# to plot the counterfactual scenario, create a data frame as if regulation
#   (the intervention) was never being implemented
datanew <- data.frame(houses=mean(data$houses),regulation=0,time=1:470/10, month=w)

# generate predictions under the counterfactual scenario and add it to the plot
pred4c <- predict(m4,datanew,type="response")/mean(data$houses)
lines(datanew$time,pred4c,col=2,lty=2)

# return the data frame to the scenario including the intervention
datanew <- data.frame(houses=mean(data$houses),regulation=rep(c(0,1),c(320,150)),
                      time= 1:470/10, month=w)

# put step-change back into the equation
m5 <- glm(str ~ regulation + time + regulation*time + harmonic(month,2,12), family = quasipoisson, data)
summary(m5)
summary(m5)$dispersion
round(ci.lin(m5,Exp=T),3)

# plot m5
par(bg = "deepskyblue4")
pred5 <- predict(m5,type="response",datanew)/mean(data$houses)
plot(data$rate,type="n",ylim=c(00,0.03),xlab="Year",ylab="Entire Home STR Ratio",
     bty="l",xaxt="n", col.lab="white", col.axis="white",cex.axis=0.85)
rect(33,0,47,0.05,col="deepskyblue3",border=F)
points(data$rate,pch=19,cex=1, col="white")
axis(1, at = ticks, labels = F, tick = TRUE)
axis(1,at=years,tick=F,labels=2015:2018,col.axis="white",cex.axis=0.85)
lines((1:470/10),pred5,col="darkorange",lwd=3)
datanew <- data.frame(houses=mean(data$houses),regulation=0,time=1:470/10, month=w)
pred5b <- predict(m5,datanew,type="response")/mean(data$houses)
lines(datanew$time,pred5b,col="darkorange",lty=3,lwd=3)
datanew <- data.frame(houses=mean(data$houses),regulation=rep(c(0,1),c(320,150)),
                      time= 1:470/10, month=w)

# F test
anova(m4,m5,test="F")

# Moving on to eviction
# histogram (confirmation of poisson distribution)
hist(data$eviction)

# plot eviction rate*100 (confirmation of seasonality)
data$rate <- with(data, eviction/houses*10^2)
plot(data$rate,type="n",ylim=c(00,0.05),xlab="Year", ylab="Eviction Rate × 100", bty="l",xaxt="n")
rect(33,0,47,0.05,col=grey(0.9),border=F)
points(data$rate[data$regulation==0],cex=0.7)
ticks <- c(9, 21, 33, 45)
axis(1, at = ticks, labels = F, tick = TRUE)
years <- c(3, 15, 27, 39)
axis(1,at=years,tick=F,labels=2015:2018)
title("San Francisco, May 2015 - March 2019")

# use the same model as m5 for eviction before policy
m6 <- glm(eviction ~ regulation + time + regulation*time + harmonic(month,2,12), family = quasipoisson, data)
summary(m6)
summary(m6)$dispersion
round(ci.lin(m6,Exp=T),3)

# plot m6
pred6 <- predict(m6,type="response",datanew)/mean(data$houses)*10^2
plot(data$rate,type="n",ylim=c(00,0.07),xlab="Year",ylab="Eviction Rate × 100",
     bty="l",xaxt="n")
rect(33,0,47,0.07,col=grey(0.9),border=F)
points(data$rate,cex=0.7)
axis(1, at = ticks, labels = F, tick = TRUE)
axis(1,at=years,tick=F,labels=2015:2018)
lines((1:470/10),pred6,col=2)
title("San Francisco, May 2015 - March 2019")
datanew <- data.frame(houses=mean(data$houses),regulation=0,time=1:470/10, month=w)
pred6b <- predict(m6,datanew,type="response")/mean(data$houses)*10^2
lines(datanew$time,pred6b,col=2,lty=2)
datanew <- data.frame(houses=mean(data$houses),regulation=rep(c(0,1),c(320,150)),
                      time= 1:470/10, month=w)

# Moving on to rent
# histogram (confirmation of poisson distribution)
hist(data$rent)

# plot rent
plot(data$rent,type="n",ylim=c(3.00,5.00),xlab="Year", ylab="Median Rent Per Square Foot", bty="l",xaxt="n")
rect(33,3,47,5,col=grey(0.9),border=F)
points(data$rent,cex=0.7)
ticks <- c(9, 21, 33, 45)
axis(1, at = ticks, labels = F, tick = TRUE)
years <- c(3, 15, 27, 39)
axis(1,at=years,tick=F,labels=2015:2018)
title("San Francisco, May 2015 - March 2019")

# plot price
plot(data$price,type="n",ylim=c(900.00,1200.00),xlab="Year", ylab="Median Home Price Per Square Foot", bty="l",xaxt="n")
rect(33,900,47,1200,col=grey(0.9),border=F)
points(data$price,cex=0.7)
ticks <- c(9, 21, 33, 45)
axis(1, at = ticks, labels = F, tick = TRUE)
years <- c(3, 15, 27, 39)
axis(1,at=years,tick=F,labels=2015:2018)
title("San Francisco, May 2015 - March 2019")

# In SF, price has a clear trend of increase while rent does not (probably due to rent control)














